import pathlib
import typing

import colourtime
import event_stream
import matplotlib
import matplotlib.colors
import PIL.Image

EXTENSION = ".png"


def run(
    input: pathlib.Path,
    output: pathlib.Path,
    begin: int,
    end: int,
    parameters: dict[str, typing.Any],
):
    if "cycle" in parameters:
        time_mapping = colourtime.generate_cyclic_time_mapping(
            duration=parameters["cycle"],
            begin=begin,
        )
    else:
        time_mapping = colourtime.generate_linear_time_mapping(begin=begin, end=end)
    with event_stream.Decoder(input) as decoder:
        image = colourtime.convert(
            begin=begin,
            end=end,
            width=decoder.width,
            height=decoder.height,
            decoder=decoder,
            colormap=matplotlib.colormaps[parameters["colormap"]],  # type: ignore
            time_mapping=time_mapping,
            alpha=parameters["alpha"],
            background_colour=matplotlib.colors.to_rgba(parameters["background_color"]),
        )
        image.resize(
            size=(
                image.width * parameters["scale"],
                image.height * parameters["scale"],
            ),
            resample=PIL.Image.Resampling.NEAREST,  # type: ignore
        ).save(
            str(output),
            compress_level=parameters["png_compression_level"],
            format="png",
        )



import importlib.resources
import pathlib
import subprocess
import typing

EXTENSION = ".svg"


def run(
    input: pathlib.Path,
    output: pathlib.Path,
    begin: int,
    end: int,
    parameters: dict[str, typing.Any],
):
    subprocess.run(
        [
            str(
                importlib.resources.files("charidotella").joinpath("assets/event_rate")
            ),
            str(input),
            str(output),
            f"--begin={begin}",
            f"--end={end}",
            f"--long={parameters['long_tau']}",
            f"--short={parameters['short_tau']}",
            f"--width={parameters['width']}",
            f"--height={parameters['height']}",
            f"--longcolor={parameters['long_tau_color']}",
            f"--shortcolor={parameters['short_tau_color']}",
            f"--axiscolor={parameters['axis_color']}",
            f"--maingridcolor={parameters['main_grid_color']}",
            f"--secondarygridcolor={parameters['secondary_grid_color']}",
        ],
        check=True,
    )



import atexit
import importlib.resources
import pathlib
import subprocess
import typing

EXTENSION = ".mp4"


def run(
    input: pathlib.Path,
    output: pathlib.Path,
    begin: int,
    end: int,
    parameters: dict[str, typing.Any],
):
    width, height = (
        int(value)
        for value in subprocess.run(
            [
                str(importlib.resources.files("charidotella").joinpath("assets/size")),
                str(input),
            ],
            check=True,
            capture_output=True,
        ).stdout.split(b"x")
    )
    width *= parameters["scale"]
    height *= parameters["scale"]
    es_to_frames_arguments = [
        str(importlib.resources.files("charidotella").joinpath("assets/es_to_frames")),
        f"--input={input}",
        f"--begin={begin}",
        f"--end={end}",
        f"--frametime={parameters['frametime']}",
        f"--scale={parameters['scale']}",
        f"--style={parameters['style']}",
        f"--tau={parameters['tau']}",
        f"--oncolor={parameters['on_color']}",
        f"--offcolor={parameters['off_color']}",
        f"--idlecolor={parameters['idle_color']}",
        f"--cumulative-ratio={parameters['cumulative_ratio']}",
    ]
    if "lambda_max" in parameters:
        es_to_frames_arguments.append(f"--lambda-max={parameters['lambda_max']}")
    if parameters["timecode"]:
        es_to_frames_arguments.append("--add-timecode")
    es_to_frames = subprocess.Popen(
        es_to_frames_arguments,
        stdout=subprocess.PIPE,
    )
    assert es_to_frames.stdout is not None
    ffmpeg = subprocess.Popen(
        [
            parameters["ffmpeg"],
            "-hide_banner",
            "-loglevel",
            "warning",
            "-stats",
            "-f",
            "rawvideo",
            "-s",
            f"{width}x{height}",
            "-framerate",
            "50",
            "-pix_fmt",
            "rgb24",
            "-i",
            "-",
            "-c:v",
            "libx264",
            "-pix_fmt",
            "yuv420p",
            "-crf",
            str(parameters["h264_crf"]),
            "-f",
            "mp4",
            "-y",
            str(output),
        ],
        stdin=subprocess.PIPE,
    )
    assert ffmpeg.stdin is not None
    frame_size = width * height * 3

    def cleanup():
        if es_to_frames is not None:
            es_to_frames.kill()
        if ffmpeg is not None:
            ffmpeg.kill()

    atexit.register(cleanup)
    while True:
        frame = es_to_frames.stdout.read(frame_size)
        if len(frame) != frame_size:
            break
        ffmpeg.stdin.write(frame)
    ffmpeg.stdin.close()
    es_to_frames.wait()
    es_to_frames = None
    ffmpeg.wait()
    ffmpeg = None
    atexit.unregister(cleanup)
